library photo_editor_pro;
import 'dart:io';
import 'dart:typed_data';
import 'package:flutter/material.dart';
import 'package:photo_editor_pro/layers/layer.dart';
import 'package:photo_editor_pro/layers/next_screen.dart';
import 'package:photo_editor_pro/module/image_item.dart';
import 'package:image_cropper/image_cropper.dart' as image_crop;
import 'package:path_provider/path_provider.dart';
import 'dart:ui' as ui;

List<File?>? imagePath;
image_crop.CroppedFile? cropped;
Uint8List? editedImageBytes;
ImageProvider? imageProvider;
MemoryImage? _memoryImage;

/// ImageSticker layer
class StickerLayer extends StatefulWidget {
  final StickerLayerData layerData;
  final Function()? onUpdate;
  final bool editable;

  const StickerLayer({
    Key? key,
    required this.layerData,
    required this.onUpdate,
    required this.editable,
  }) : super(key: key);

  @override
  State<StickerLayer> createState() => _StickerLayerState();
}

class _StickerLayerState extends State<StickerLayer> {
  bool isFrameVisible = false;
  @override
  Widget build(BuildContext context) {
    imageProvider = _getImageProvider(widget.layerData.image);

    if (imageProvider == null) {
      return const SizedBox();
    }
    return Positioned(
      left: widget.layerData.offset.dx,
      top: widget.layerData.offset.dy,
      child: GestureDetector(
        onTap: () async {
          print("imageProvider=====${imageProvider.runtimeType}");
          ImageItem imageItem = widget.layerData.image;
          print("imageItem=====${imageItem.runtimeType}");
          final result = await Navigator.push(context, MaterialPageRoute(
            builder: (context) {
              return NextScreen(imageData: imageItem.bytes);
            },
          ));
         if(result != null){
           print("result=====${result.runtimeType}");
           await _cropImage(result);
         }
        },
        onPanUpdate: (details) {
          if (widget.editable) {
            widget.layerData.offset += details.delta;
            widget.onUpdate?.call();
          }
        },
        onDoubleTap: () {
          setState(() {
            isFrameVisible = !isFrameVisible;
          });
        },
        child: Stack(
          children:[
            Opacity(
              opacity: widget.layerData.opacity,
              child: Transform.rotate(
                angle: widget.layerData.rotation,
                child: Transform.scale(
                  scale: widget.layerData.scale,
                  child: Image(
                    image: imageProvider!,
                    height: 150,
                  ),
                ),
              ),
            ),
            if (isFrameVisible)
              Container(
                height: 150,
                width: 92,
                decoration: BoxDecoration(
                  border: Border.all(
                    color: Colors.red,
                    width: 2.0,
                  ),
                ),
              ),
          ]
        ),
      ),
    );
  }

  ImageProvider _getImageProvider(dynamic imageData) {
    if (imageData is ImageItem) {
      return MemoryImage(imageData.bytes);
    } else {
      throw ArgumentError('Invalid image data type: $imageData');
    }
  }

  Future<void> _loadAndCropImage() async {
    ImageItem imageItem = widget.layerData.image;
    // Convert Uint8List to File
    Uint8List bytes = imageItem.bytes;
    Directory tempDir = await getTemporaryDirectory();
    String tempPath = tempDir.path;
    File file = File('$tempPath/${imageItem.hashCode}.png');
    await file.writeAsBytes(bytes);
    imagePath = [file];

    // Crop image
    image_crop.ImageCropper imageCropper = image_crop.ImageCropper();
    image_crop.CroppedFile? cropped = await imageCropper.cropImage(
      sourcePath: imagePath![0]!.path,
      aspectRatioPresets: [
        image_crop.CropAspectRatioPreset.square,
        image_crop.CropAspectRatioPreset.ratio3x2,
        image_crop.CropAspectRatioPreset.original,
        image_crop.CropAspectRatioPreset.ratio4x3,
        image_crop.CropAspectRatioPreset.ratio16x9,
      ],
      uiSettings: [
        image_crop.AndroidUiSettings(
          cropFrameColor: Colors.red,
          activeControlsWidgetColor: Colors.green,
          toolbarTitle: 'Crop',
          cropGridColor: Colors.blue,
          initAspectRatio: image_crop.CropAspectRatioPreset.original,
          lockAspectRatio: false,
          backgroundColor: Colors.white,
        ),
        image_crop.IOSUiSettings(
          title: 'Crop',
        )
      ],
    );

    if (cropped != null) {
      Uint8List croppedBytes = await File(cropped.path).readAsBytes();
      imageItem.bytes = croppedBytes;
      widget.onUpdate?.call();
      print('Image cropped successfully: ${cropped.path}');
    } else {
      print('Image cropping canceled');
    }
  }

  Future<void> _cropImage(result) async {
    final recorder = ui.PictureRecorder();
    final imageSize = Size(result.width.toDouble(), result.height.toDouble());
    final canvas = Canvas(recorder, Rect.fromPoints(Offset.zero, Offset(imageSize.width, imageSize.height)));
    // Draw the cropped image onto the canvas
    final paint = Paint()..filterQuality = FilterQuality.high;
    canvas.drawImageRect(
      result,
      Rect.fromPoints(Offset.zero, Offset(result.width.toDouble(), result.height.toDouble())),
      Rect.fromPoints(Offset.zero, Offset(imageSize.width, imageSize.height)),
      paint,
    );
    // Convert the canvas to an ImageProvider
    final picture = recorder.endRecording();
    final image = await picture.toImage(imageSize.width.toInt(), imageSize.height.toInt());
    final ByteData? byteData = await image.toByteData(format: ui.ImageByteFormat.png);
    final Uint8List uint8List = byteData!.buffer.asUint8List();
    _memoryImage = MemoryImage(uint8List);

    setState(() {
      imageProvider = _memoryImage;
    });

    // Update the image data in the layerData
    if (widget.layerData.image is ImageItem) {
      (widget.layerData.image as ImageItem).bytes = uint8List;
    }
    widget.onUpdate?.call();
  }


}
